#include <stdlib.h>
#include <stdio.h>
#include "mkl_lapacke.h" // C-wrapper to the Frotran LApack lib.
#include "mkl.h"
#include <time.h>

/* Main program */
int main(int argc, char *argv[] ) {
  if (argc!=4) {
    printf("This program expects 3 command-line arguments: matrix-size random-or-sparse-matrix(0 or 1)  mkl_num_threads\n.");
    exit(0);
  }
  
  const int N=atoi(argv[1]);
  const int LDA=N;
  const int random=atoi(argv[2]);
  mkl_set_num_threads(atoi(argv[3]));
  printf("Matrix size=%i\n",N);	
  printf("Number of threads=%i\n",mkl_get_max_threads());	

  /* Some working space needed by the lapack/mkl routine */
  MKL_INT n = N, lda = LDA, info,i,j;
  double w[N];
  MKL_Complex16* a; a=malloc(N*LDA*sizeof(MKL_Complex16));
  
  /* Intializes random number generator */
  srand(999); // No randomness, for reproducibility
  if (random==1) {
    /* dense random matrix */
    printf("Random Hermitian matrix.\n");
    for (i=0; i < n; i++ )
      for(j = 0; j<n; j++ )
	a[i*lda+j].real=rand()/n,a[i*lda+j].imag=rand()/n;
  } else {
    printf("Simple sparse 3-band matrix.\n");
    //Simple 3-bands matrix => eigenvalues = 2*cos(k_i) with k_i=i*2*pi/n and i=integer
    for (i=0; i < n; i++ ) {
      int j=(i+1); if (j>=n) j-=n;
      a[i*lda+j].real=1;a[i*lda+j].imag=0;
      j=i-1;if (j<0) j+=n;
      a[i*lda+j].real=1;a[i*lda+j].imag=0;
      j=i;
      a[i*lda+j].real=0;a[i*lda+j].imag=0;
    }
  }
  /* Variable used for timing */
  struct timespec start, finish;
  double elapsed;

  /* Solve eigenproblem */
  // LAPACKE_zheevd: computes all eigenvalues and eigenvectors of a complex Hermitian
  // matrix A using divide and conquer algorithm

  clock_gettime(CLOCK_MONOTONIC, &start);
  info = LAPACKE_zheevd( LAPACK_ROW_MAJOR, 'V', 'L', n, a, lda, w );
  clock_gettime(CLOCK_MONOTONIC, &finish);
  /*Timing*/
  elapsed = (finish.tv_sec - start.tv_sec);
  elapsed += (finish.tv_nsec - start.tv_nsec) / 1000000000.0;
  /* Check for convergence */
  if( info > 0 ) {
    printf( "The algorithm failed to compute eigenvalues.\n" );
    exit( 1 );
  }
  /* Print the extreme eigenvalues */
  printf("Smallest eigen value=%6.2f\n",w[0]); 
  printf("Largest value=%6.2f\n",w[n-1]); 
  printf("Elaspe time (s): %g\n",elapsed);

  FILE *fp;
  fp=fopen("timing.dat", "a");
  fprintf(fp,"%d\t%g\t%d\n",n,elapsed,mkl_get_max_threads());

}
