#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <omp.h>
#include <cblas.h>

/* Variable used for timing */
struct timespec start, finish;
double elapsed;


int main(int argc, char *argv[]) {
if (argc!=2) {
	printf("This program expects 1 command-line argument: matrix-size\n.");
	exit(0);
}
int N,i,j;long int N2;
N=atoi(argv[1]);N2=N*N;
printf ("Initializing the matrices ...");
//Memory allocation for the arrays:
double *A, *B, *C;
A = (double *)malloc( N2*sizeof( double ) );
B = (double *)malloc( N2*sizeof( double ) );
C = (double *)malloc( N2*sizeof( double ) );

#pragma omp parallel
{
#pragma omp for nowait
    for (i = 0; i < (N2); i++) A[i] = (double)(i+1);
#pragma omp for nowait
    for (i = 0; i < (N2); i++) B[i] = (double)(-i-1);
#pragma omp for
    for (i = 0; i < (N2); i++) C[i] = 0.0;
}
printf (" done.\n");
printf ("Computing matrix product using OpenBLAS dgemm function via CBLAS interface...");
clock_gettime(CLOCK_MONOTONIC, &start);
cblas_dgemm(CblasRowMajor, CblasNoTrans, CblasNoTrans, 
		N, N, N, 1.0, A, N, B, N, 1.0, C, N);
clock_gettime(CLOCK_MONOTONIC, &finish);
printf (" done.\n");
/*Compute the elapsed time*/
elapsed = (finish.tv_sec - start.tv_sec);
elapsed += (finish.tv_nsec - start.tv_nsec) / 1000000000.0;
printf("Elaspe time (s): %g\n",elapsed); 
FILE *fp;
fp=fopen("timing.dat", "a");
fprintf(fp,"%d\t%g\t%d\n",N,elapsed,omp_get_max_threads());

long int nflop=N2*(N+N-1);
double gflops=nflop/elapsed/1e9;
printf("%g GFlops\n",gflops); 
free(A); free(B); free(C);
return 0;
}


