#include <stdlib.h>
#include <stdio.h>
#include <omp.h>
#include <lapacke.h> // C-wrapper to the Frotran LApack lib.

int main(int argc, char *argv[] ) {
if (argc!=2) {
	printf("This program expects 1 command-line argument: matrix-size\n.");
	exit(0);
}

const int N=atoi(argv[1]);
const int LDA=N;
printf("Matrix size=%i\n",N);	
printf("Number of threads=%i\n",omp_get_max_threads());	

/* Some working space needed by the lapack/mkl routine */
int n = N, lda = LDA, info,i,j;
double w[N];
lapack_complex_double* a; a=malloc(N*LDA*sizeof(lapack_complex_double));

srand(999); // No randomness, for reproducibility
/* dense random matrix */
printf("Random Hermitian matrix.\n");
for (i=0; i < n; i++ )
for(j = 0; j<n; j++ )
a[i*lda+j]=lapack_make_complex_double(rand(),rand());

// LAPACKE_zheevd: computes all eigenvalues and eigenvectors of a complex Hermitian
// matrix A using divide and conquer algorithm

info = LAPACKE_zheevd( LAPACK_ROW_MAJOR, 'V', 'L', n, a, lda, w );

/* Check for convergence */
if( info > 0 ) {
	printf( "The algorithm failed to compute eigenvalues.\n" );
	exit( 1 );
}
/* Print the extreme eigenvalues */
printf("Smallest eigen value=%6.2f\n",w[0]); 
printf("Largest value=%6.2f\n\n",w[n-1]); 
}

